@@ -94,7 +94,7 @@ def _cleanup_files():
 def get_distutils_distro(_cache=[]):
     if _cache:
         return _cache[0]
-    # late import to accomodate for setuptools override
+    # late import to accommodate for setuptools override
     from distutils.dist import Distribution
     distutils_distro = Distribution()
 
@@ -116,6 +116,7 @@ def get_distutils_distro(_cache=[]):
 EXT_DEP_MODULES = {
     'tag:numpy':    'numpy',
     'tag:pythran':  'pythran',
+    'tag:setuptools':  'setuptools.sandbox',
     'tag:asyncio':  'asyncio',
     'tag:pstats':   'pstats',
     'tag:posix':    'posix',
@@ -371,6 +372,7 @@ VER_DEP_MODULES = {
     (3,4,999): (operator.gt, lambda x: x in ['run.initial_file_path',
                                              ]),
     (3,5): (operator.lt, lambda x: x in ['run.py35_pep492_interop',
+                                         'run.py35_asyncio_async_def',
                                          'run.mod__spec__',
                                          'run.pep526_variable_annotations',  # typing module
                                          ]),
@@ -1064,13 +1066,13 @@ class CythonCompileTestCase(unittest.TestCase):
     def _match_output(self, expected_output, actual_output, write):
         try:
             for expected, actual in zip(expected_output, actual_output):
-                self.assertEquals(expected, actual)
+                self.assertEqual(expected, actual)
             if len(actual_output) < len(expected_output):
                 expected = expected_output[len(actual_output)]
-                self.assertEquals(expected, None)
+                self.assertEqual(expected, None)
             elif len(actual_output) > len(expected_output):
                 unexpected = actual_output[len(expected_output)]
-                self.assertEquals(None, unexpected)
+                self.assertEqual(None, unexpected)
         except AssertionError:
             write("\n=== Expected: ===\n")
             write('\n'.join(expected_output))
@@ -1765,7 +1767,7 @@ def main():
                       help="do not delete the generated C files (allows passing --no-cython on next run)")
     parser.add_option("--no-cleanup-sharedlibs", dest="cleanup_sharedlibs",
                       action="store_false", default=True,
-                      help="do not delete the generated shared libary files (allows manual module experimentation)")
+                      help="do not delete the generated shared library files (allows manual module experimentation)")
     parser.add_option("--no-cleanup-failures", dest="cleanup_failures",
                       action="store_false", default=True,
                       help="enable --no-cleanup and --no-cleanup-sharedlibs for failed tests only")
